@@ -9,13 +9,6 @@ import phylotreelib as pt
 import numpy as np
 import pandas as pd
 
-# Python note: I could make these imports conditional (placing them in MBASR section)
-import rpy2
-import rpy2.robjects as robjects
-from rpy2.robjects.packages import importr
-from rpy2.robjects import pandas2ri
-from rpy2.robjects.conversion import localconverter
-
 ###################################################################################################
 ###################################################################################################
 
@@ -36,38 +29,72 @@ class AncRecon:
     (100% for leaves, possibly less for internal nodes)
 
     Object can output information about nodes, branches, changes, etc.
+
+    Internal node IDs are set to be the same as in the anctrait object
+
+    NOTE: in state of transition. Currently does not get seqprob
     """
 
     def __init__(self, ancseq, anctrait):
-        self.tree = ancseq.tree
-        self.alignment = ancseq.alignment
-        if hasattr(ancseq, "seqprob"):
-            self.seqprob = ancseq.seqprob
-        else:
-            self.seqprob = None # Python note: should I set to zeroes?
+        self.tree = anctrait.tree
+        self.traitdict = anctrait.traitdict
+        self.traitprob = anctrait.traitprob    # Python note: Always there?
+
         self.sortedintnodes = sorted(list(self.tree.intnodes))
         self.sortednodes = sorted(list(self.tree.leaves))
         self.sortednodes.extend(self.sortedintnodes)
 
-        trait_tree = anctrait.tree
-        orig_traitdict = anctrait.traitdict
-        orig_traitprob = anctrait.traitprob    # Python note: Always there?
-
-        self.traitdict, self.traitprob = self._match_traitid_seqid(self.tree, trait_tree,
-                                                            orig_traitdict, orig_traitprob)
-
-    ###########################################################################################
-
-    def _match_traitid_seqid(self, seqtree, trait_tree, traitdict, traitprob):
-        trateid2seqid,unmatch1,unmatch2 = trait_tree.match_nodes(seqtree)
-        if (unmatch1 != None) or (unmatch2 != None):
+        seqid2traitid, u1, u2 = ancseq.tree.match_nodes(self.tree)
+        if (u1 != None) or (u2 != None):
             raise AncError("Seq-tree and trait-tree are rooted differently. Cannot merge info")
-        new_traitdict = {}
-        new_traitprob = {}
-        for trateid,seqid in trateid2seqid.items():
-            new_traitdict[seqid] = traitdict[trateid]
-            new_traitprob[seqid] = traitprob[trateid]
-        return new_traitdict, new_traitprob
+
+        self.alignment = sequencelib.Seq_alignment(seqtype=ancseq.alignment.seqtype)
+        for seq in ancseq.alignment:
+            seqid = ancseq.name2id[seq.name]  # Python note: not this objects responsibility!!!
+            newname = str(seqid2traitid[seqid])
+            seq.rename(newname)
+            self.alignment.addseq(seq)
+
+    ###############################################################################################
+
+    def write_tree(self, outfilename, label="branchtype", br=1):
+        """Prints Nexus tree to outfilename, where branch labels carry information:
+
+        label="branchtype": labels give same inforamtion as branchtype in branchinfo and
+        branchdiff, namely traits for parent, nodefrom, nodeto (e.g., human-swine-swine)
+
+        label="nodeid": nodeid used in other output functions (This means labels should
+        be interpreted as belonging to the child node, not the branch)"""
+
+        out_tree = self.tree.copy_treeobject(copylabels=False)
+
+        if label == "branchtype":
+            for nodefrom in out_tree.intnodes:
+                for nodeto in out_tree.children(nodefrom):
+                    traitfrom = self.traitdict[nodefrom]
+                    traitto = self.traitdict[nodeto]
+                    if br==1:
+                        branch_type = f"{traitfrom}-{traitto}"
+                    elif br==2:
+                        if nodefrom != self.tree.root:
+                            parent = self.tree.parent(nodefrom)
+                            traitfrom_parent = self.traitdict[parent]
+                        else:
+                            traitfrom_parent = "None"
+                        branch_type = f"{traitfrom_parent}-{traitfrom}-{traitto}"
+                    out_tree.setlabel(nodefrom, nodeto, branch_type)
+            treestring = out_tree.nexus()
+        elif label == "nodeid":
+            # Hack: add label for root manually
+            out_tree.set_nodeid_labels()
+            treestring = out_tree.nexus()
+            rootid = str(out_tree.root)
+            treestring = treestring.replace(");", f"){rootid};")
+        else:
+            raise AncError("'label' must be 'branchtype' or 'nodeid'")
+
+        with open(outfilename, "w") as outfile:
+            outfile.write(treestring)
 
     ###############################################################################################
 
@@ -114,23 +141,28 @@ class AncRecon:
 
     def branchinfo(self, outfilename, varseq=False, poslist=None, zeroindex=True,
                     printif_traitdiff=False, printif_seqdiff=False, probmin=None,
-                    translate=False):
+                    translate=False, br=1):
         """Writes results to 'outfilename':
         Outputs one line per branch with following informations:
-          nodeid_from  nodeid_to  trait_from  trait_to  traitprob_from  traitprob_to seq_from  seq_to
-        Option varseq=True (default) output only variable sites from sequences.
-        Option poslist: specify sites to be printed.
-        Option zeroindex=True: Start indexing of poslist at 0 (otherwise start at 1)
-        Option printif_traitdiff=True: only print branches where traits differ
-        Option printif_seqdiff=True: only print branches where selected residues have changed
-        option probmin (if not None): Only print info where both nodes have traitprob > probmin
-        option translate: Translate DNA to amino acid sequences. Note position info is now for aa
+          nodeid_from  nodeid_to  branchlen trait_from  trait_to
+                                            traitprob_from  traitprob_to seq_from  seq_to
+                                            branchtype
+
+        varseq=True (default) output only variable sites from sequences.
+        poslist: specify sites to be printed.
+        zeroindex=True: Start indexing of poslist at 0 (otherwise start at 1)
+        printif_traitdiff=True: only print branches where traits differ
+        printif_seqdiff=True: only print branches where selected residues have changed
+        probmin (if not None): Only print info where both nodes have traitprob > probmin
+        translate: Translate DNA to amino acid sequences. Note position info is now for aa
+        br=1: branchtype 1st degree traits: node_from to nodeto (hu-sw)
+        br=2: branchtype 2nd degree traits: parent to node_from to nodeto (hu-hu-sw)
         """
 
         with open(outfilename, "w") as outfile:
-            outfile.write("# {}\t{}\t{}\t{}\t{}\t{}\t{}\t{}\n".format(
-                   "node_from", "node_to", "trait_from", "trait_to",
-                   "traitprob_from", "traitprob_to", "seq_from", "seq_to"))
+            outfile.write("# {}\t{}\t{}\t{}\t{}\t{}\t{}\t{}\t{}\t{}\n".format(
+                   "node_from", "node_to", "branch_len", "trait_from", "trait_to",
+                   "traitprob_from", "traitprob_to", "seq_from", "seq_to", "branch_type"))
             pos = None
             if varseq and poslist:
                 raise AncError("Specify either varseq or poslist option - not both")
@@ -153,8 +185,18 @@ class AncRecon:
                             (self.traitprob[nodeto] > probmin)):
                         seqnamefrom = str(nodefrom)
                         seqnameto = str(nodeto)
+                        blen = self.tree.nodedist(nodefrom, nodeto)
                         traitfrom = self.traitdict[nodefrom]
                         traitto = self.traitdict[nodeto]
+                        if br==1:
+                            branch_type = f"{traitfrom}-{traitto}"
+                        elif br==2:
+                            if nodefrom != self.tree.root:
+                                parent = self.tree.parent(nodefrom)
+                                traitfrom_parent = self.traitdict[parent]
+                            else:
+                                traitfrom_parent = "None"
+                            branch_type = f"{traitfrom_parent}-{traitfrom}-{traitto}"
                         traitprobfrom = self.traitprob[nodefrom]
                         traitprobto = self.traitprob[nodeto]
                         if pos:
@@ -171,18 +213,19 @@ class AncRecon:
                         if printif_seqdiff and (seqfrom==seqto):
                             printbranch = False
                         if printbranch:
-                            outfile.write("{}\t{}\t{}\t{}\t{:.3f}\t{:.3f}\t{}\t{}\n".format(
-                                   nodefrom, nodeto, traitfrom, traitto,
-                                   traitprobfrom, traitprobto, seqfrom, seqto))
+                            outfile.write("{}\t{}\t{}\t{}\t{}\t{:.3f}\t{:.3f}\t{}\t{}\t{}\n".format(
+                                   nodefrom, nodeto, blen, traitfrom, traitto,
+                                   traitprobfrom, traitprobto, seqfrom, seqto, branch_type))
 
     ###############################################################################################
 
     def branchdiff(self, outfilename, zeroindex=True, printif_traitdiff=False, probmin=None,
-                   translate=False):
+                   translate=False, br=1):
         """Writes results to 'outfilename':
         Output is one line of output for each sequence change, on all branches of the tree:
-            node_from, node_to, site, trait_from, trait_to,
-                                      traitprob_from, traitprob_to, residue_from, residue_to
+            node_from, node_to, branch_len, site, trait_from, trait_to,
+                                      traitprob_from, traitprob_to, residue_from, residue_to,
+
         Here 'site' is the index of the sequence residue.
         Option zeroindex=False causes numbering to start at 1 (otherwise at 0)
         Option printif_traitdiff=True: only print branches where traits differ
@@ -191,9 +234,9 @@ class AncRecon:
         """
 
         with open(outfilename, "w") as outfile:
-            outfile.write("# {}\t{}\t{}\t{}\t{}\t{}\t{}\t{}\t{}\n".format(
-               "node_from", "node_to", "seqpos", "trait_from", "trait_to",
-               "traitprob_from", "traitprob_to", "residue_from", "residue_to"))
+            outfile.write("# {}\t{}\t{}\t{}\t{}\t{}\t{}\t{}\t{}\t{}\t{}\n".format(
+               "node_from", "node_to", "branch_len", "seqpos", "trait_from", "trait_to",
+               "traitprob_from", "traitprob_to", "residue_from", "residue_to", "branch_type"))
             if translate:
                 alignment = self.alignment.translate()
             else:
@@ -205,8 +248,18 @@ class AncRecon:
                             (self.traitprob[nodeto] > probmin)):
                         seqnamefrom = str(nodefrom)
                         seqnameto = str(nodeto)
+                        blen = self.tree.nodedist(nodefrom, nodeto)
                         traitfrom = self.traitdict[nodefrom]
                         traitto = self.traitdict[nodeto]
+                        if br==1:
+                            branch_type = f"{traitfrom}-{traitto}"
+                        elif br==2:
+                            if nodefrom != self.tree.root:
+                                parent = self.tree.parent(nodefrom)
+                                traitfrom_parent = self.traitdict[parent]
+                            else:
+                                traitfrom_parent = "None"
+                            branch_type = f"{traitfrom_parent}-{traitfrom}-{traitto}"
                         traitprobfrom = self.traitprob[nodefrom]
                         traitprobto = self.traitprob[nodeto]
                         seqfrom = alignment.getseq(seqnamefrom)
@@ -216,9 +269,9 @@ class AncRecon:
                         # Could test for traitdiff before setting seq, but neater code this way...
                         if (not printif_traitdiff) or (traitfrom!=traitto):
                             for site,resfrom,resto in difflist:
-                                outfile.write("{}\t{}\t{}\t{}\t{}\t{:.3f}\t{:.3f}\t{}\t{}\n".format(
-                                   nodefrom, nodeto, site, traitfrom, traitto,
-                                   traitprobfrom, traitprobto, resfrom, resto))
+                                outfile.write("{}\t{}\t{}\t{}\t{}\t{}\t{:.3f}\t{:.3f}\t{}\t{}\t{}\n".format(
+                                   nodefrom, nodeto, blen, site, traitfrom, traitto,
+                                   traitprobfrom, traitprobto, resfrom, resto, branch_type))
 
 ###################################################################################################
 ###################################################################################################
@@ -226,7 +279,7 @@ class AncRecon:
 class BasemlSeq:
     """Class representing parser for BASEML rst file.
 
-    Methods for extracing tree, alignment, and residue probabilities for ancestral
+    Methods for extracting tree, alignment, and residue probabilities for ancestral
     (and contemporaneous) sequences.
 
     Tree is constructed based on branch info (87..88 87..89 etc.) and newick string.
@@ -392,104 +445,8 @@ class BasemlSeq:
 ###################################################################################################
 ###################################################################################################
 
-class MBASRTrait:
-    """Class representing parser for MBASR ancestral reconstruction file.
-
-    Methods for extracing one tree and one alignment of ancestral (and contemporaneous) sequences.
-
-    Tree is constructed based on branch info (87..88 87..89 etc.) and newick string.
-    Internal nodes are numbered according to branch info. Leaves are named from Newick string.
-
-    Ancestral sequences (corresponding to internal nodes on tree) are named based on nodeIDs.
-    The method also returns
-    """
-
-    def __init__(self, treefile, intnodefile, leafstatefile, state0, state1):
-        """Read files, store as dataframes, ready for further parsing.
-        Tree is read as dataframe (so we have info about internal nodeIDs):
-            parent, node, branch.length,	label
-        Intnode reconstructions are read as dataframe:
-            nodeid, pstate1, pstate2
-        Leafnode states read as dataframe:
-            leafnodeid, state
-        """
-
-        # Read treefile, store branch info as pandas dataframe
-        # Note: I am cargo culting the rpy2 code here...
-        ape = importr("ape")
-        tidytree = importr("tidytree")
-        tree = ape.read_tree(treefile)
-        dftreeR = tidytree.as_tibble_phylo(tree)
-        with localconverter(robjects.default_converter + pandas2ri.converter):
-            self.dftreePy = robjects.conversion.rpy2py(dftreeR)
-        self.nseq = self.dftreePy.shape[0]
-
-        # Read internal node ancestral reconstruction state info
-        self.intnodestate = pd.read_csv(intnodefile, delim_whitespace=True,
-                                        header=None, names=["nodeID","pstate0","pstate1"])
-        self.nintnode = self.intnodestate.shape[0]
-
-        # Read leaf node state info
-        self.leafnodestate = pd.read_csv(leafstatefile, delim_whitespace=True,
-                                        header=None, names=["leafID","state"])
-        self.nleaf = self.leafnodestate.shape[0]
-        self.state0 = state0
-        self.state1 = state1
-        self.tree = self.get_tree()
-        self.traitdict, self.traitprob = self.get_trait_dict()
-
-    ###########################################################################################
-
-    def get_tree(self):
-        parentlist = []
-        childlist = []
-
-        for i in range(self.nseq):
-            parent, node, label = self.dftreePy.iloc[i,[0,1,3]]
-            # Tidy tree df has one row where root is both parent and node: Discard.
-            if parent != node:
-                parentlist.append(parent)
-                # Tidytree parent is either leafname (type: string) or
-                # NA_character_ (type: rpy2.rinterface_lib.sexp.NACharacterType)
-                if type(label) == str:
-                    childlist.append(label)
-                else:
-                    childlist.append(node)
-        tree = pt.Tree.from_branchinfo(parentlist, childlist)
-        return tree
-
-    ###########################################################################################
-
-    def get_trait_dict(self):
-        traitnamedict = {0:self.state0, 1:self.state1}
-        nodeidlist = list(self.leafnodestate["leafID"])
-        intnodeidlist = [int(x.replace("node","")) for x in self.intnodestate["nodeID"]]
-        nodeidlist.extend(intnodeidlist)
-
-        traitstatelist = [traitnamedict[trait] for trait in self.leafnodestate["state"]]
-        traitproblist = [1.0] * self.nleaf
-        intnodestatelist = []
-        intproblist = []
-        for i in range(self.nintnode):
-            p0, p1 = self.intnodestate.iloc[i,[1,2]]
-            if p0 > 0.5:
-                intnodestatelist.append(self.state0)
-                intproblist.append(p0)
-            else:
-                intnodestatelist.append(self.state1)
-                intproblist.append(p1)
-        traitstatelist.extend(intnodestatelist)
-        traitproblist.extend(intproblist)
-
-        traitdict = dict(zip(nodeidlist, traitstatelist))
-        traitprobdict = dict(zip(nodeidlist, traitproblist))
-
-        return (traitdict, traitprobdict)
-
-###################################################################################################
-###################################################################################################
-
 class _TreeTime:
+    """Baseclass - do not instantiate"""
 
     def _parsetreefile(self, treefile):
         with pt.Nexustreefile(treefile) as tf:
@@ -510,7 +467,7 @@ class _TreeTime:
                 name2id[nodename] = child
         # Add nodename for root (which was listed at end of treestring)
         # a bit hackish, since I know there is a nodename and a branchlen: NODE_0000000:0.00100
-        rootname = re.sub(":[0-9]\.[0-9]+", "", tree.belowroot)
+        rootname = re.sub(":[0-9]\.[0-9]+", "", tree.below_root)
         name2id[rootname] = tree.root
         return name2id
 
@@ -520,19 +477,22 @@ class _TreeTime:
 class TreeTimeSeq(_TreeTime):
 
     def __init__(self, ancseqfile, seq_treefile):
-        self.tree, self.seqname2id = self._parsetreefile(seq_treefile)
-        self.alignment = self._parsealignfile(ancseqfile, self.tree, self.seqname2id)
+        self.tree, self.origseqname2id = self._parsetreefile(seq_treefile)
+        self.alignment, self.name2id = self._parsealignfile(ancseqfile, self.tree, self.origseqname2id)
 
     ###########################################################################################
 
-    def _parsealignfile(self, alignfile, tree, name2id):
+    def _parsealignfile(self, alignfile, tree, origname2id):
         """Reads alignment; returns alignment with nodes renamed to match nodes on seqtree"""
         sf = sequencelib.Seqfile(alignfile)
         alignment = sf.read_alignment()
+        name2id = {}
         for origname in alignment.seqnamelist.copy():
-            newname = str(name2id[origname])
+            newid = origname2id[origname]
+            newname = str(newid)
+            name2id[newname] = newid
             alignment.changeseqname(origname, newname)
-        return alignment
+        return alignment, name2id
 
 ###################################################################################################
 ###################################################################################################
@@ -547,7 +507,7 @@ class TreeTimeTrait(_TreeTime):
     ###########################################################################################
 
     def _parsetraits(self, trait_treefile, trait_probfile, trait_tree, traitname2id):
-        traitdict = self._parse_annot_tree(trait_treefile, traitname2id) # This dict has no root entry
+        traitdict = self._parse_annot_tree(trait_treefile, traitname2id)
         traitprob, traitid2AB = self._parse_CSV(trait_probfile, traitname2id)
         traitdict = self._match_AB_trait(traitid2AB, traitdict, trait_tree)   # Add root trait. Check consistency
         return traitdict, traitprob
@@ -569,7 +529,6 @@ class TreeTimeTrait(_TreeTime):
                     treestring += line
         treestring = treestring.replace("\n", "")
         nodenames = re.findall(r"[\(\),]([^\(\),:]+):", treestring)
-        nodenames = nodenames[:-1]    # One too long: no trait for root node for some reason
         traits = re.findall(r'\[&[\w]+=\"(\w+)\"\]', treestring)
 
         traitdict = {}
@@ -613,9 +572,6 @@ class TreeTimeTrait(_TreeTime):
         else:
             AB2trait["A"] = other_trait
             AB2trait["B"] = traitval
-        root_ABtrait = traitid2AB[trait_tree.root]
-        root_trait = AB2trait[root_ABtrait]
-        traitdict[trait_tree.root] = root_trait
         self._check_trait_consistency(AB2trait, traitid2AB, traitdict)
         return traitdict
 
